// test_iambic.v

/*

IambicV: Iambic keyer in Verilog
Copyright  2011 Stephen C. Trier

This file is part of IambicV

IambicV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

IambicV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with IambicV.  If not, see <http://www.gnu.org/licenses/>.

*/


`timescale 1 us / 1 us
`define CLOCK 1000
`define SPEED 5
`define DAH (3*`SPEED)
`define DIT (`SPEED)
`define SP (`SPEED)
`define SP3 (3*`SPEED)

module testbench;
    reg rst_n;
    reg clk;
    reg [7:0] speed;
    reg modeb;
    reg dit;
    reg dah;
    wire key;
    reg keyExpected;

    reg passed;

    task kg8ih();
    begin
            // Dadidah (K)
            // Test dit insertion with a very brief, very early tap of the dit paddle
            fork
                begin   // Dah paddle
                    repeat (`DAH + `SP + `DIT + `SP + `DAH) @(negedge clk) dah = 1;
                    @(negedge clk) dah = 0;
                end
                begin   // Dit paddle
                    @(negedge clk);
                    @(negedge clk) dit = 1;
                    @(negedge clk) dit = 0;
                end
                begin   // Key Expected
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP3) @(posedge clk) keyExpected = 0;
                end
            join


            // Dadadit (G)
            // 
            // Mode B: Check dit insertion.  Let go of the dah paddle as early as possible,
            // then tap the dit paddle after letting go of dah but before the second dah is over.
            //
            // Mode A: Play it straight.
            fork
                begin   // Dah paddle
                    repeat (`DAH + `SP + 1) @(negedge clk) dah = 1;
                    @(negedge clk) dah = 0;
                end
                begin   // Dit paddle
                    repeat (`DAH + `SP + `DIT) @(negedge clk);
                    if (modeb)
                        repeat (`DIT) @(negedge clk) dit = 1;
                    else
                        repeat (`DAH + `SP) @(negedge clk) dit = 1;
                    repeat (`DIT) @(negedge clk) @(negedge clk) dit = 0;
                end
                begin   // Key Expected
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP3) @(posedge clk) keyExpected = 0;
                end
            join

            // Dadadadidit (8)
            // Let go of the dah paddle as early as possible, then tap the dit paddle after
            // letting go of dah but before the second dah is over
            fork
                begin   // Dah paddle
                    repeat (2 * (`DAH + `SP) + `DIT) @(negedge clk) dah = 1;
                    @(negedge clk) dah = 0;
                end
                begin   // Dit paddle
                    repeat (2 * (`DAH + `SP) + 1) @(negedge clk) dit = 0;
                    repeat (`DAH + `SP) @(negedge clk) dit = 1;
                    repeat (`DIT + `SP + 2) @(negedge clk) dit = 1;
                    dit = 0;
                end
                begin   // Key Expected
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DAH) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP3) @(posedge clk) keyExpected = 0;
                end
            join

            // Two dits (I)
            // Hold the dit paddle until after both dits are done
            fork
                begin   // Dit paddle
                    repeat (`DIT + `SP + `DIT + `SP) @(negedge clk) dit = 1;
                    dit = 0;
                end
                begin   // Key Expected
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP3) @(posedge clk) keyExpected = 0;
                end
            join


            // Four dits (H)
            fork
                begin   // Dit paddle
                    repeat (3 * (`DIT + `SP) + `DIT) @(negedge clk) dit = 1;
                    dit = 0;
                end
                begin   // Key Expected
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP) @(posedge clk) keyExpected = 0;
                    repeat (`DIT) @(posedge clk) keyExpected = 1;
                    repeat (`SP3) @(posedge clk) keyExpected = 0;
                end
            join
        end
    endtask // kg8ih

    iambic i(rst_n, clk, speed, modeb, dit, dah, key);

    always #(`CLOCK/2) clk = !clk;

    always @(negedge clk)
    begin
        if (key != keyExpected)
            passed = 0;
    end

    initial
    begin
        passed = 1;
        clk = 0;
        rst_n = 0;
        speed = `SPEED - 1;
        modeb = 1;
        dit = 0; dah = 0; keyExpected = 0;

        // Reset
        repeat (2) @(negedge clk) rst_n = 0;
        repeat (3) @(negedge clk) rst_n = 1;
        @(posedge clk);

        // Test basic operation in mode B
        modeb = 1;
        kg8ih();

        // Add a little delay after the word to keep the timing diagram easier to read
        repeat (5 * `SP) @(posedge clk);

        // Test basic operation in mode A
        modeb = 0;
        kg8ih();

        // Add a delay after the word.
        repeat (5 * `SP) @(posedge clk);
        
        // Now look at the difference between mode A and mode B
        // One difference happens when one tries to send "C" (dadidadit).
        // Someone used to mode B who sends in mode A will miss the last dit (dadidah)
        // Someone used to mode A who sends in mode B will add an extra dit dadidadidit

        // C sent for mode B, in mode B
        // Test dit insertion with a very brief, very early tap of the dit paddle
        modeb = 1;
        fork
            begin   // Dah paddle
                repeat (`DAH + `SP + `DIT + `SP + `DIT) @(negedge clk) dah = 1;
                @(negedge clk) dah = 0;
            end
            begin   // Dit paddle
                @(negedge clk) dit = 0;
                repeat (`DAH + `SP + `DIT + `SP + `DIT) @(negedge clk) dit = 1;
                @(negedge clk) dit = 0;
            end
            begin   // Key Expected
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DIT) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DIT) @(posedge clk) keyExpected = 1;
                repeat (`SP3) @(posedge clk) keyExpected = 0;
            end
        join

        // C sent for mode B, in mode A
        // Test dit insertion with a very brief, very early tap of the dit paddle
        modeb = 0;
        fork
            begin   // Dah paddle
                repeat (`DAH + `SP + `DIT + `SP + `DAH) @(negedge clk) dah = 1;
                @(negedge clk) dah = 0;
            end
            begin   // Dit paddle
                @(negedge clk);
                repeat (`DAH + `SP + `DIT + `SP + `DAH) @(negedge clk) dit = 1;
                @(negedge clk) dit = 0;
            end
            begin   // Key Expected
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DIT) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP3) @(posedge clk) keyExpected = 0;
            end
        join

        // C sent for mode A, in mode A
        modeb = 0;
        fork
            begin   // Dah paddle
                repeat (`DAH + `SP + `DIT + `SP + `DAH) @(negedge clk) dah = 1;
                @(negedge clk) dah = 0;
            end
            begin   // Dit paddle
                @(negedge clk);
                repeat (`DAH + `SP + `DIT + `SP + `DAH + `SP + 1) @(negedge clk) dit = 1;
                @(negedge clk) dit = 0;
            end
            begin   // Key Expected
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DIT) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DAH) @(posedge clk) keyExpected = 1;
                repeat (`SP) @(posedge clk) keyExpected = 0;
                repeat (`DIT) @(posedge clk) keyExpected = 1;
                repeat (`SP3) @(posedge clk) keyExpected = 0;
            end
        join

        // Write text to alert the user of our status.  The passed variable will also help.
        if (passed)
            $write("test_iambic.v tests passed");
        else
            $write("test_iambic.v tests FAILED");

    end



endmodule
